#include <fork.h>
#include <io.h>
/* 
See: Practical PRAM Programming with Fork95, A Tutorial
by C. Kessler. Nr. 97-2. pag 16.
"Self-Scheduling for independent loop iterations"
implementation by casiano@ull.es
*/

/* test the results with different processor numbers and 
	different iteration numbers

sh int work[] = { 1, 2, 3, 4, 
									5, 6, 7, 8, 
									9, 10, 11, 12, 
									13, 14, 15, 16 }; 
sh int work[] = { 1, 10, 20, 30, 
								 40, 50, 60, 70,	
								80, 90, 100, 110,
								120, 130, 140, 150, 160}; 
*/
sh int work[] = { 1, 100, 200, 300, 
								 400, 500, 600, 700,	
								800, 900, 1000, 1100,
								1200, 1300, 1400, 1500, 1600}; 
sh int N, chunk = 1;
sh int slowest, iteration = 0;

void execute_iteration(int i) {
	int j;

	for(j=0;j<work[i];j++) ;
}

main(void) {
	int i, k, last;
	unsigned int localtime, starttime;

	if ($ == 0) {
		printf("\nEnter loop size: ");
		scanf("%d",&N);
		printf("\n%d processors. Times for N= %d iterations ...\n",__STARTED_PROCS__,N);
	}
	barrier;
	starttime = getct();
	for (i=__PROC_NR__; i < N; i += __STARTED_PROCS__)
		execute_iteration(i);
	localtime = getct()-starttime;
	syncmax(&slowest,localtime);
	pprintf("cyclic scheduling: local time = %d\n", localtime);
	barrier;
	if ($ == 0) { 
		pprintf("cyclic scheduling: maximum time = %d\n", slowest);
		pprintf("************************************************\n");
	}
	barrier;

	slowest = 0;
	barrier;
	starttime = getct();
	for (i=mpadd(&iteration,1); i < N; i=mpadd(&iteration,1))
		execute_iteration(i);
	localtime = getct()-starttime;
	syncmax(&slowest,localtime);
	pprintf("self-scheduling local time = %d\n", localtime);
	barrier;
	if ($ == 0) 
		pprintf("self scheduling: maximum time = %d\n", slowest);
	barrier;

	if ($ == 0) {
		printf("\nEnter chunk size: ");
		scanf("%d",&chunk);
		printf("\nTimes for N= %d iterations chunk = %d ...\n",N,chunk);
	}
	/* static cyclic scheduling with chunk policy */
	slowest = 0;
	barrier;
	starttime = getct();
	for (i=$*chunk; i < N; i+=__STARTED_PROCS__*chunk) {
		last = i+chunk;
		for(k=i;k<last;k++)
			execute_iteration(k);
	}
	/* barrier;  testing!!! */
	localtime = getct()-starttime;
	syncmax(&slowest,localtime);
	pprintf("static cyclic scheduling with chunk = %d local time = %d\n", chunk, localtime);
	barrier;
	if ($ == 0) 
		pprintf("static scheduling with chunk = %d: maximum time = %d\n", chunk, slowest);
	barrier; /* to avoid contamination of the next 
							assignment to slowest */
	/* self-scheduling with chunk policy */
	slowest = 0;
	iteration = 0;
	barrier;
	starttime = getct();
	for (i=mpadd(&iteration,chunk); i < N; i=mpadd(&iteration,chunk)) {
		last = i+chunk;
		for(k=i;k<last;k++)
			execute_iteration(k);
	}
	/* barrier;  testing!!! */
	localtime = getct()-starttime;
	syncmax(&slowest,localtime);
	pprintf("self-scheduling with chunk = %d local time = %d\n", chunk, localtime);
	barrier;
	if ($ == 0) 
		pprintf("self scheduling with chunk = %d: maximum time = %d\n", chunk, slowest);
	barrier;
}
